## Install packages
if(!require("FuzzyLandscapes")) {
  if(!require('devtools')) install.packages('devtools')
  devtools::install_git('https://gitlab.com/CRC1266-A2/FuzzyLandscapes.git')
}

if(!require("raster")) {
  install.packages("raster")
}

## load libraries
library(FuzzyLandscapes)
library(raster)

## plot figures?
plot_figures <- TRUE
if(plot_figures == TRUE) {
  if(!require("magrittr")) {
    install.packages("magrittr")
  }
  library(magrittr)

  if(!require("tibble")) {
    install.packages("tibble")
  }
  library(tibble)
  
  if(!require("ggplot2")) {
    install.packages("ggplot2")
  }
  library(ggplot2)
  
  if(!require("cowplot")) {
    install.packages("cowplot")
  }
  library(cowplot)

}

## show plots?
show_fs_plots <- FALSE

sourcedir <- file.path(".","data","raw_data")
targetdir<- file.path(".","data","derived_data")

### Fuzzyfing distance to rivers for agriculture, arboriculture and pasture
## From Popa & Knitter 2015: "Since slope magnitude strongly influences soil and water movements, and thus
## erosion, it represents a limiting factor for environment utilization. Results of empirical
## analyses allow us to classify slope in classes that link its value to potential human
## practices. Below 5 % (2.86°) slope, areas are suitable for agricultural purposes and
## erosion is a minor problem. Pixels corresponding to this characteristic belong to the
## fuzzy class small. The critical slope for construction is 8 % (4.57°), since at this point,
## erosion increases significantly. The fuzzy class moderate covers pixels up to this
## threshold. The fuzzy class high refers to slope values higher than 8 % and particularly
## above 10 % (5.7°), when erosion becomes a severe problem and land utilization is only
## possible with large efforts (empirical values based on Cooke and Doornkamp 1974, p.
## 361; Leser 1968, p. 38; Pécsi 1985)."
slope <- raster::raster(file.path(targetdir, "slope.tif"))
agr_slope <- fl_create_ras(
  method="trapezoid",
  rast = slope,
  p1 = 0, 
  p2 = 0, 
  p3 = 2.86, 
  p4 = 5.7, 
  setname = "agr_slope")

if(plot_figures == TRUE) {
  agr_slope_fs <- tibble(
    type = "Agriculture (slope in degree)",
    range = c(0, 0.0, 2.8, 5.0, 30.0),
    fuzzy = c(0, 1.0, 1.0, 0.0,  0.0)) %>%    
    ggplot(aes(x = range, y = fuzzy)) +
    geom_line() +
    theme_minimal(base_family = "Source Sans Pro") +
    labs(y = "",
         x = "\nSlope (in degree)\n")
}

if(show_fs_plots == TRUE) {
  plot_fs_comb(agr_slope, xyrange=c(1,30))
  plot_fs(agr_slope)
}

agr_slope_high_labour <- fl_create_ras(
  method="trapezoid",
  rast = slope,
  p1 = 0, 
  p2 = 0, 
  p3 = 5.7, 
  p4 = 10.0, 
  setname = "agr_slope_high_labour")

if(plot_figures == TRUE) {
  agr_slope_high_labour_fs <- tibble(
    type = "Agriculture high labour scenario (slope in degree)",
    range = c(0, 0.0, 5.7, 10.0, 30.0),
    fuzzy = c(0, 1.0, 1.0, 0.0,  0.0)) %>%    
    ggplot(aes(x = range, y = fuzzy)) +
    geom_line() +
    theme_minimal(base_family = "Source Sans Pro") +
    labs(y = "",
         x = "\nSlope (in degree; high labour scenario)\n")
}

if(show_fs_plots == TRUE) {
  plot_fs_comb(agr_slope_high_labour_fs, xyrange=c(1,30))
  plot_fs(agr_slope_high_labour_fs)
}

### Fuzzyfing TWI for agriculture, arboriculture, pasture and woodland
twi <- raster::raster(file.path(targetdir, "twi.tif"))
agr_twi <- fl_create_ras(method="trapezoid",
                         rast = twi,
                         p1 = 6.0, 
                         p2 = 7.0, 
                         p3 = 16.0, 
                         p4 = 25.0,
                         setname = "agr_twi")

if(show_fs_plots == TRUE) {
  plot_fs_comb(agr_twi, xyrange=c(0.0,20.0))
  plot_fs(agr_twi)
}

if(plot_figures == TRUE) {
  agr_twi_fs <- tibble(
    type = "Agriculture (Topographic Wetness Index)",
    range = c(0, 6.0, 7.0, 16.0, 25.0),
    fuzzy = c(0, 1.0, 1.0,  0.0,  0.0)) %>%  
    ggplot(aes(x = range, y = fuzzy)) +
    geom_line() +
    theme_minimal(base_family = "Source Sans Pro") +
    labs(y = "",
         x = "\nTopographic Wetness Index\n")
}

## Write suitability raster Variant
## ==================================================
## two_8hday
export_raster <- FuzzyLandscapes::fl_combine_fuzzy(
  agr_twi,
  agr_slope,
  raster::raster("./data/derived_data/two_8hday.tif"),
  combinator = "and")

raster::writeRaster(export_raster$FuzzyRaster,
                    filename = file.path(targetdir, "suitability_agr_2_two8h.tif"),
                    format = "GTiff",
                    overwrite = TRUE)

export_raster <- FuzzyLandscapes::fl_combine_fuzzy(
  agr_twi,
  agr_slope_high_labour,
  raster::raster("./data/derived_data/two_8hday.tif"),
  combinator = "and")

raster::writeRaster(export_raster$FuzzyRaster,
                    filename = file.path(targetdir, "suitability_agr_5_high_labour_two8h.tif"),
                    format = "GTiff",
                    overwrite = TRUE)

## Chora
export_raster <- FuzzyLandscapes::fl_combine_fuzzy(
  agr_twi,
  agr_slope,
  raster::raster("./data/derived_data/chora_ras.tif"),
  combinator = "and")

raster::writeRaster(export_raster$FuzzyRaster,
                    filename = file.path(targetdir, "suitability_agr_1_chora.tif"),
                    format = "GTiff",
                    overwrite = TRUE)

export_raster <- FuzzyLandscapes::fl_combine_fuzzy(
  agr_twi,
  agr_slope_high_labour,
  raster::raster("./data/derived_data/chora_ras.tif"),
  combinator = "and")

raster::writeRaster(export_raster$FuzzyRaster,
                    filename = file.path(targetdir, "suitability_agr_4_high_labour_chora.tif"),
                    format = "GTiff",
                    overwrite = TRUE)

## Micro_Region_Catchment_Based catchment
export_raster <- FuzzyLandscapes::fl_combine_fuzzy(
  agr_twi,
  agr_slope,
  raster::raster("./data/derived_data/micro_region_catchment_based_ras.tif"),
  combinator = "and")

raster::writeRaster(export_raster$FuzzyRaster,
                    filename = file.path(targetdir, "suitability_agr_3_micro_region_catchment_based.tif"),
                    format = "GTiff",
                    overwrite = TRUE)

export_raster <- FuzzyLandscapes::fl_combine_fuzzy(
  agr_twi,
  agr_slope_high_labour,
  raster::raster("./data/derived_data/micro_region_catchment_based_ras.tif"),
  combinator = "and")

raster::writeRaster(export_raster$FuzzyRaster,
                    filename = file.path(targetdir, "suitability_agr_6_high_labour_micro_region_catchment_based.tif"),
                    format = "GTiff",
                    overwrite = TRUE)



## Create figures
## ==================================================
library(cowplot)

fuzzy_sets <- ggdraw() +
  draw_plot(agr_slope_fs,
            x = 0,
            y = 0,
            width = 1/3-.01,
            height = 1) +
  draw_plot(agr_slope_high_labour_fs,
            x = 1/3,
            y = 0,
            width = 1/3-.01,
            height = 1) +  
  draw_plot(agr_twi_fs,
            x = 2/3,
            y = 0,
            width = 1/3-.01,
            height = 1) +
  draw_plot_label(label = c("A", "B", "C"),
                  x = c(0, 1/3, 2/3),
                  y = c(1, 1, 1),                        
                  hjust = 0,
                  family = "Source Sans Pro",
                  size = 12
                  )

#plot(fuzzy_sets)

ggsave(plot = fuzzy_sets,
       filename = "Paper/Figures/fuzzy_sets.pdf",
       device = cairo_pdf,
       width = 20,
       height = 6,
       scale = 1.3,
       units = "cm")
