# -*- coding: utf-8 -*-
"""
POPULATION/LABOR ESTIMATIONS based on land-use assumptions

Estimates the farm population by checking the needed farming labor (in labor 
days (Ld)) for the harvest periode based on ethnographic/historical/ data on 
agricultural activities from the so called informed  by the roman writer 
Columella (after Sherwood et al. 2020) and the work of Halstead (2014).

"""

import os
import sys
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.patheffects as pe


csfont = {'fontname':'Source Sans Pro'}
hfont = {'fontname':'Source Sans Pro'}

# Get and set workdirectory
print(os.getcwd())

# insert your working direction path if the above is not where you want to work
workdir = './'

inp_dir = "./data/raw_data/"
out_dir = "./data/derived_data/"

# Change workdirectory
os.chdir(workdir)
print('%s uses working directory %s' % (sys.argv[0], workdir))

# -----------------------------------------------------------------------------
### LOAD AND PREPARE DATA

# Load CSV containing the area requirements per capita for caloric need between
# 1400 to 3200 kcal per day
df = pd.read_csv(out_dir+"kcal_change.csv", sep=";", decimal=",", 
                 index_col="kcal_req")

# Create a dataframe holding the statistical values of mean, standard 
# deviation, standard error for each given population size
df_stats = pd.DataFrame(columns = ['pop_size',
                                   'bar_mea','whe_mea','cer_mea',
                                   'pea_mea','bea_mea','chi_mea','len_mea',
                                   'fla_mea'])

# Create a sequence of a range of population sizes of interest
step = 1000
pop_min = 20000
pop_max = 200000+step
pop_size_arr = np.arange(pop_min,pop_max,step)

# Specify range of daily needed kcal in which stats should be calculated
start = 2100
end = 2100

# Write stats for each land-use category into the df
for pop in pop_size_arr:
    df_stats = df_stats.append({'pop_size':pop,
                                'bar_mea':np.round(
                                    (df.loc[start:end,"a_bar"] * \
                                          pop).mean(),5),
                                'whe_mea':np.round(
                                    (df.loc[start:end,"a_bar"] * \
                                          pop).mean(),5),
                                'cer_mea':np.round(
                                    ((df.loc[start:end,"a_bar"] + \
                                      df.loc[start:end,"a_whe"]) * \
                                          pop).mean(),5),
                                'pea_mea':np.round(
                                    ((df.loc[start:end,"a_pea"]) * \
                                          pop).mean(),5),
                                'bea_mea':np.round(
                                    ((df.loc[start:end,"a_fav"]) * \
                                          pop).mean(),5),
                                'chi_mea':np.round(
                                    ((df.loc[start:end,"a_chi"]) * \
                                          pop).mean(),5),
                                'len_mea':np.round(
                                    ((df.loc[start:end,"a_len"]) * \
                                          pop).mean(),5),
                                'fla_mea':np.round(
                                    (df.loc[start:end,"a_fla"]*pop).mean(),5)},
                               ignore_index=True)
  

### ---------------------------------------------------------------------------
### HARVEST WORKLOAD

# Load CSV containing workhour estimations for harvest, threshing and 
# winnowing
lab_harvest = pd.read_csv(inp_dir+"labor_harvest.csv", sep=",",
                          decimal=",")
lab_threshing = pd.read_csv(inp_dir+"labor_threshing.csv", sep=",",
                            decimal=",")
lab_winnowing = pd.read_csv(inp_dir+"labor_winnowing.csv", sep=",",
                            decimal=",")

# Create dataframe for storing labor estimations
df_harvest = pd.DataFrame(index = ["Ld/ha"]) 

# Calculating Ld/ha for harvest cereals, leguminosae and flax. 
df_harvest["cereals"] = \
    lab_harvest["Ld/ha"][lab_harvest["plant_type"] == "cereal"].mean()

df_harvest["pea_mea"] = \
    lab_harvest["Ld/ha"][lab_harvest["plant"] == "chickpea"].mean()
df_harvest["bea_mea"] = \
    lab_harvest["Ld/ha"][lab_harvest["plant"] == "beans"].mean()
df_harvest["chi_mea"] = \
    lab_harvest["Ld/ha"][lab_harvest["plant"] == "chickpea"].mean()
df_harvest["len_mea"] = \
    lab_harvest["Ld/ha"][lab_harvest["plant"] == "lentil"].mean()
  

# Calculating Ld/ha for threshing an winnowing cereals    
df_harvest["bar_thr"] = lab_threshing["Ld/kg"].mean() * 770.0 #LandUseQuantifieR mean yield for barley
df_harvest["whe_thr"] = lab_threshing["Ld/kg"].mean() * 625.0 #LandUseQuantifieR mean yield for common wheat

df_harvest["bar_win"] = lab_winnowing["Ld/kg"].mean() * 770.0 #LandUseQuantifieR mean yield for barley
df_harvest["whe_win"] = lab_winnowing["Ld/kg"].mean() * 625.0 #LandUseQuantifieR mean yield for common wheat

#Export dataframe for further use 
df_harvest.to_csv(out_dir+"harvest_Ld_per_ha.csv")


# Calculate and plot  needed labour force (proportion of given populations) 
# for a given duration harvest time (here 5-7 weeks)

# Array holding labour days needed to harvest given cultivated areas
labour_need = (df_stats["cer_mea"].values * df_harvest["cereals"].values) + \
      (df_stats["pea_mea"].values * df_harvest["pea_mea"].values) + \
      (df_stats["bea_mea"].values * df_harvest["bea_mea"].values) + \
      (df_stats["chi_mea"].values * df_harvest["chi_mea"].values) + \
      (df_stats["len_mea"].values * df_harvest["len_mea"].values) + \
      (df_stats["bar_mea"].values * df_harvest["bar_thr"].values) + \
      (df_stats["whe_mea"].values * df_harvest["whe_thr"].values) + \
      (df_stats["bar_mea"].values * df_harvest["bar_win"].values) + \
      (df_stats["whe_mea"].values * df_harvest["whe_win"].values)  

# Harvest period in days
harvest_days= np.arange(27,51,1)

# Calculate proportion of population needed to harvest for each amount of days
# given as harvest period.
pop_pro = []
for i in harvest_days:
    pop_pro.append((labour_need / (df_stats["pop_size"] * i))[0])

y_limitation = np.round((labour_need / (df_stats["pop_size"] * np.min(harvest_days)))[0],2)

# Plot
plt.figure(figsize=(12, 6))
plt.box(False)
plt.plot(harvest_days, pop_pro, linestyle="-", color="black")
#plt.hlines(y=0.0,xmin=26.75,xmax=50.25,color='grey', alpha=0.5, linewidth=3)
plt.grid()
plt.fill_between([28,35], [0,0], [y_limitation,y_limitation], color='black', alpha=0.2,
                 interpolate=True)
plt.fill_between([35,42], [0,0], [y_limitation,y_limitation], color='black', alpha=0.1,
                 interpolate=True)
plt.fill_between([42,49], [0,0], [y_limitation,y_limitation], color='black', alpha=0.2,
                 interpolate=True)
plt.text(31.5,0.05, "week 5", fontsize=16, ha='center', va='center',
         path_effects=[pe.withStroke(linewidth=4, foreground="w")]) 
plt.text(38.5,0.05, "week 6", fontsize=16, ha='center', va='center',
         path_effects=[pe.withStroke(linewidth=4, foreground="w")]) 
plt.text(45.5,0.05, "week 7", fontsize=16, ha='center', va='center',
         path_effects=[pe.withStroke(linewidth=4, foreground="w")]) 
plt.yticks(size=12)
plt.xticks(np.arange(27,51,1), size=12)
plt.tick_params(axis='both', which='both', length=0)
plt.xlim(np.min(harvest_days)-0.25,np.max(harvest_days)+0.25)
plt.ylim(-0.01,y_limitation)
plt.ylabel("Proportion of farming population", size=16, labelpad=10)
plt.xlabel("Harvest period in days", size=16, labelpad=10)
plt.savefig(out_dir+"harvest_labour.png",dpi=400)

