# -*- coding: utf-8 -*-
"""CARRYING CAPACITY ESTIMATON

This script evaluates the carrying capacity of a landscape (raster) based on 
the needed area to cultivate crops/leguminosae, as main diet share, by
comparing it against the available area, that is available under consideration
of different suitability thresholds. It is possible to add fallow land
fractions to enlarge the sceanrio.

The results are dataframes (and heatmaps of them) giving the proportion of the
population that can be sustained by the, as suitable defined, area for 
cultivation. Values of >= 1.0 mean that the carrying capacity of the landscape 
is not met, no or little  subsistence stress can be assumed. Values < 1.0 
suggest that under the assumed scenario the landscpae cannot sustain the 
population. Strategies to overcome this can be an increase of cultivated area 
or trade. 

Land-use area estimations and the fuzzyfication - suitability mapping - of the 
landscape need to be performed in R with the "LandUseQuantifieR"
(https://zenodo.org/record/3476674) respectively "FuzzyLandscapes"
(https://zenodo.org/record/1747005) packages.

"""

import os
import sys
from osgeo import gdal
import numpy as np
import pandas as pd

import matplotlib.pyplot as plt
import matplotlib as mpl
mpl.use('cairo')
import seaborn as sns

# Get and set workdirectory
print(os.getcwd())

# Insert your working direction path if the above is not where you want to work
workdir = './'

inp_dir = "./data/raw_data/"
out_dir = "./data/derived_data/"

# Change workdirectory
os.chdir(workdir)
print('%s uses working directory %s' % (sys.argv[0], workdir))

   
### --------------------------------------------------------------------------
### PREPATE GEODATA FOR CALCULATING CARRYING CAPACITIES

# Load river GEOTIFF containing stream bed information, original transformation,
# projection and for suitability manipulation.
x_in = gdal.Open(out_dir+"suitability_agr_1_chora.tif")

# Extract transformation and projectionparameters.
out_gt = x_in.GetGeoTransform()
out_proj = x_in.GetProjection()

# Calculate the factor to translate area in ha to count of cell of the used
# map resultion.
cell_sqm = out_gt[1]**2.0
ha_to_cell = 10000.0 / cell_sqm

# Delete not needed GDAl objects
del x_in

### --------------------------------------------------------------------------
### CALCULATING CARRYING CAPACITIES

# Load CSV containing the area requirements per capita for caloric need between
# 1400 to 3200 kcal per day
df = pd.read_csv(out_dir+"kcal_change.csv", sep=";", decimal=",", 
                 index_col="kcal_req")

# Create a dataframe holding the statistical values of mean, standard 
# deviation, standard error for each given population size
df_stats = pd.DataFrame(columns = ['pop_size','cro_mea','leg_mea'])

# Create a sequence of a range of population sizes of interest
step = 1000
pop_min = 20000
pop_max = 200000+step
pop_size_arr = np.arange(pop_min,pop_max,step)

# Specify range of daily needed kcal in which stats should be calculated
start = 2100
end = 2100

# Write stats for each land-use category into the df
for pop in pop_size_arr:
    df_stats = df_stats.append({'pop_size':pop,
                                'cro_mea':np.round(
                                    ((df.loc[start:end,"a_bar"] + \
                                      df.loc[start:end,"a_whe"]) * \
                                          pop).mean(),5),
                                'leg_mea':np.round(
                                    ((df.loc[start:end,"a_len"] + \
                                     df.loc[start:end,"a_pea"] + \
                                     df.loc[start:end,"a_fav"] + \
                                      df.loc[start:end,"a_chi"]) * \
                                          pop).mean(),5)},
                               ignore_index=True)


# Search for all masked suitabilty raster (here for agriculture) in a given
# directory, to load them and store them in a list object.
file_lst = os.listdir(out_dir)

# Specify suitabilty raster by its "prefix"(here "suitability_agr") and write
# only those in a list object
word = "suitability_agr"
name_lst = []
for i in file_lst:
    if word in i:
        name_lst.append(i)
    else:
        pass

# Sort to gain right order of suitability rasters
name_lst.sort(reverse=False)

# Load suitability raster present in the name_lst
suit_lst = []
for i in np.arange(0,len(name_lst)):
    arr_in = gdal.Open(out_dir+name_lst[i])
    arr = np.float32(np.array(arr_in.GetRasterBand(1).ReadAsArray()))
    arr = np.ma.masked_array(arr, np.isnan(arr))
    suit_lst.append(arr)


# Calculates the fraction of the area that is provided by cells of a given 
# suitability threshold for each given populations size and under consideration 
# of different fallow systems. This is repeated for all masked suitability
# ratser.
# Keep in mind: already +30% of the area of crops are clutivated for 
# storage and seed for the next year (LandUseQuantifieR default).

# Create list object to store the dataframes of each suitability ratser's 
# scenario of fallowing
df_super_lst = []

for a in np.arange(0,len(suit_lst)):
    
    # Specify % additional land to represent fallow
    fallow_frac = [0.0,0.5,1.0]
    # Create list object to store the dataframes of each fallow scenario
    df_lst = []
    
    # Iterate over fallow scenarios
    for frac in fallow_frac:
        # Create dataframe to store current fallow sceanrio for each given
        # population size and all given suitability thresholds (0.1-1.0)
        df_fracs_fallow = pd.DataFrame(columns = ['pop_size', 
                                    0.0,0.1,0.2,0.3,0.4,0.5,0.6,0.7,0.8,0.9,1.0,
                                    'scenario'])
        df_fracs_fallow = df_fracs_fallow.astype(float)
        df_fracs_fallow ['pop_size'] = df_stats['pop_size']
        df_fracs_fallow ['scenario'] = name_lst[a][12:-4]
                    
        # Iterate over all given population sizes
        for j in list(df_fracs_fallow.columns)[1:-1]:
            # Extract all cells of the suitability raster (a) >= the 
            # suitability threshold (j) translate them into ha measures
            # and divde by the needed ha needed for each given population
            # size (i)
            df_fracs_fallow.loc[::,j] = len(suit_lst[a][suit_lst[a] >= j]) / \
                ha_to_cell / (
                (df_stats.loc[::,"cro_mea"] + df_stats.loc[::,"leg_mea"]) +
                ((df_stats.loc[::,"cro_mea"] + df_stats.loc[::,"leg_mea"]) 
                * frac))
                    
                    
        # Reindex the datafame       
        df_fracs_fallow = df_fracs_fallow.set_index('pop_size')
        # Append the dataframe into the df_lst object; but change the order of
        # population sizes so lowest is at teh bottom and highest on top
        df_lst.append(df_fracs_fallow.iloc[::-1,::])
        # Save the datafame to CSV 
        df_fracs_fallow.to_csv(out_dir+"df_fracs_fallow_"
                    +str(int(frac*100))+"_"+name_lst[a][12:-4]+"_"+".csv")
        
        # # Plot the datafram as heatmap. The value of 1. "scratching the carrying 
        # # capacity is used as the center for the colormap.
        # plt.figure(figsize=(12, 6))
        # sns.heatmap(df_fracs_fallow.loc[::-1,0.1:1.0], cmap='PRGn', center=1,
        #             annot=False, vmin=0, vmax=2)
        # plt.xticks(size=12)
        # plt.yticks(size=12)
        # plt.ylabel("Population size", size=16, labelpad=10)
        # plt.xlabel("Suitability values", size=16, labelpad=10)
        # plt.title("Carrying capacity estimation for "+name_lst[a][18:-4]+" with +"+str(int(frac*100))+"% fallow land", 
        #           size=20, pad=20)
        # plt.savefig(out_dir+"heatmap_carrying_capacity_frac_fallow_" +str(int(frac*100))+"_"+name_lst[a][18:-4]+"_"+".png", dpi=400,format="png")
    
    # Append the df_lst object, containig the three fallow fraction sceanrios
    # of each suitability raster into the df_super_lst
    df_super_lst.append(df_lst)   


# Plot CCI for the "Chora"[0], "Two days distance catchment"[1] and the 
# "Bakırçay catchment"[2] normal labor input scenario (numbers in [] refer to 
# the index the respectiv data frames hold in the df_super_lst).

# Define colormap to use
cmap = mpl.cm.PRGn

fig = plt.figure(figsize=(12, 7))

plt.subplot(3, 3, 1)
plt.subplots_adjust(left=None, bottom=0.1, right=0.825, top=0.9, 
                wspace=0.1, hspace=0.3)

sns.heatmap(df_super_lst[0][0].loc[::,0.1:1.0], cbar=None, cmap=cmap, center=1,
            annot=False,
            yticklabels=25,
            xticklabels=1)
plt.contour(df_super_lst[0][0].loc[::,0.1:1.0], levels=[1.0], origin = "lower",
            colors="black", linestyles="--")
plt.grid(alpha=0.33)
plt.yticks(size=11)
plt.title("Chora\n \n1a - No fallow land", size=12, pad=5)
plt.ylabel("")
plt.tick_params(axis='x', which='both', labelbottom=False, rotation=0)

plt.subplot(3, 3, 2)
sns.heatmap(df_super_lst[1][0].loc[::,0.1:1.0], cbar=None, cmap=cmap, center=1,
            annot=False,
            yticklabels=25,
            xticklabels=1)
plt.contour(df_super_lst[1][0].loc[::,0.1:1.0], levels=[1.0], origin = "lower",
            colors="black", linestyles="--")
plt.grid(alpha=0.33)
plt.title("Micro-Region\n \n2a - No fallow land", size=12, pad=5)
plt.ylabel("")
plt.tick_params(axis='y', which='both', labelleft=False, labelright=False,
                rotation=0, right=False, left=True)
plt.tick_params(axis='x', which='both', labelbottom=False, rotation=0)

plt.subplot(3, 3, 3)
sns.heatmap(df_super_lst[2][0].loc[::,0.1:1.0], cbar=None, cmap=cmap, center=1,
            annot=False,
            yticklabels=25,
            xticklabels=1)
plt.contour(df_super_lst[2][0].loc[::,0.1:1.0], levels=[1.0], origin = "lower",
            colors="black", linestyles="--")
plt.grid(alpha=0.33)
plt.title("Catchments\n \n3a - No fallow land", size=12, pad=5)
plt.ylabel("")
plt.tick_params(axis='y', which='both', labelleft=False, labelright=False,
                rotation=0, right=False, left=True)
plt.tick_params(axis='x', which='both', labelbottom=False, rotation=0)

plt.subplot(3, 3, 4)
sns.heatmap(df_super_lst[0][1].loc[::,0.1:1.0], cbar=None, cmap=cmap, center=1,
            annot=False,
            yticklabels=25,
            xticklabels=1)
plt.contour(df_super_lst[0][1].loc[::,0.1:1.0], levels=[1.0], origin = "lower",
            colors="black", linestyles="--")
plt.grid(alpha=0.33)
plt.title("1b - 1:2 fallow land", size=12, pad=5)
plt.ylabel("Population size", size=16, labelpad=10)
plt.yticks(size=11)
plt.tick_params(axis='x', which='both', labelbottom=False, rotation=0)

plt.subplot(3, 3, 5)
sns.heatmap(df_super_lst[1][1].loc[::,0.1:1.0], cbar=None, cmap=cmap, center=1,
            annot=False,
            yticklabels=25,
            xticklabels=1)
plt.contour(df_super_lst[1][1].loc[::,0.1:1.0], levels=[1.0], origin = "lower",
            colors="black", linestyles="--")
plt.grid(alpha=0.33)
plt.title("2b - 1:2 fallow land", size=12, pad=5)
plt.ylabel("")
plt.tick_params(axis='y', which='both', labelleft=False, labelright=False,
                rotation=0, right=False, left=True)
plt.tick_params(axis='x', which='both', labelbottom=False, rotation=0)

plt.subplot(3, 3, 6)
sns.heatmap(df_super_lst[2][1].loc[::,0.1:1.0], cbar=None, cmap=cmap, center=1,
            annot=False,
            yticklabels=25,
            xticklabels=1)
plt.contour(df_super_lst[2][1].loc[::,0.1:1.0], levels=[1.0], origin = "lower",
            colors="black", linestyles="--")
plt.grid(alpha=0.33)
plt.title("3b - 1:2 fallow land", size=12, pad=5)
plt.ylabel("")
plt.tick_params(axis='y', which='both', labelleft=False, labelright=False,
                rotation=0, right=False, left=True)
plt.tick_params(axis='x', which='both', labelbottom=False, rotation=0)

plt.subplot(3, 3, 7)
sns.heatmap(df_super_lst[0][2].loc[::,0.1:1.0], cbar=None, cmap=cmap, center=1,
            annot=False,
            yticklabels=25,
            xticklabels=1)
plt.contour(df_super_lst[0][2].loc[::,0.1:1.0], levels=[1.0], origin = "lower",
            colors="black", linestyles="--")
plt.yticks(size=11)
plt.xticks(size=11)
plt.grid(alpha=0.33)
plt.title("1c - 1:1 fallow land", size=12, pad=5)
plt.ylabel("")

plt.subplot(3, 3, 8)
sns.heatmap(df_super_lst[1][2].loc[::,0.1:1.0], cbar=None, cmap=cmap, center=1,
            annot=False,
            yticklabels=25,
            xticklabels=1)
plt.contour(df_super_lst[1][2].loc[::,0.1:1.0], levels=[1.0], origin = "lower",
            colors="black", linestyles="--")
plt.grid(alpha=0.33)
plt.xticks(size=11)
plt.title("2c - 1:1 fallow land", size=12, pad=5)
plt.ylabel("")
plt.xlabel("Suitability threshold", size=16, labelpad=10)
plt.tick_params(axis='y', which='both', labelleft=False, labelright=False,
                rotation=0, right=False, left=True)

plt.subplot(3, 3, 9)
sns.heatmap(df_super_lst[2][2].loc[::,0.1:1.0], cbar=None, cmap=cmap, center=1,
            annot=False,
            yticklabels=25,
            xticklabels=1)
plt.contour(df_super_lst[2][2].loc[::,0.1:1.0], levels=[1.0], origin = "lower",
            colors="black", linestyles="--")
plt.grid(alpha=0.33)
plt.xticks(size=11)
plt.title("3c - 1:1 fallow land", size=12, pad=5)
plt.ylabel("")
plt.tick_params(axis='y', which='both', labelleft=False, labelright=False,
                rotation=0, right=False, left=True)

cbar_ax = fig.add_axes([.85, .1, .03, 0.8])
norm = mpl.colors.Normalize(vmin=0, vmax=2)
cb1 = mpl.colorbar.ColorbarBase(cbar_ax, cmap=cmap, norm=norm,
                                orientation='vertical')
cb1.set_label('Carrying capacity index ($CCI$)', size=16, labelpad=30, rotation=270)

# fig.suptitle('Carrying capacity estimation', size=20)
plt.savefig(workdir+"Paper/Figures/heatmap_CCI_norm_labour.png",dpi=600)
#plt.show()


# Plot CCI for the "Chora"[3], "Two days distance catchment"[4] and the 
# "Bakırçay catchment"[5] high labor input scenario (numbers in [] refer to 
# the index the respectiv data frames hold in the df_super_lst).

fig = plt.figure(figsize=(12, 7))

plt.subplot(3, 3, 1)
plt.subplots_adjust(left=None, bottom=0.1, right=0.825, top=0.9, 
                wspace=0.1, hspace=0.3)

sns.heatmap(df_super_lst[3][0].loc[::,0.1:1.0], cbar=None, cmap=cmap, center=1,
            annot=False,
            yticklabels=25,
            xticklabels=1)
plt.contour(df_super_lst[3][0].loc[::,0.1:1.0], levels=[1.0], origin = "lower",
            colors="black", linestyles="--")
plt.grid(alpha=0.33)
plt.yticks(size=11)
plt.title("Chora\n \n1a - No fallow land", size=12, pad=5)
# plt.ylabel("Population size", size=16, labelpad=10)
# plt.xticks(size=12)
# plt.yticks(size=12)
plt.ylabel("")
plt.tick_params(axis='x', which='both', labelbottom=False, rotation=0)

plt.subplot(3, 3, 2)
sns.heatmap(df_super_lst[4][0].loc[::,0.1:1.0], cbar=None, cmap=cmap, center=1,
                annot=False,
            yticklabels=25,
            xticklabels=1)
plt.contour(df_super_lst[4][0].loc[::,0.1:1.0], levels=[1.0], origin = "lower",
            colors="black", linestyles="--")
plt.grid(alpha=0.33)
plt.title("Micro-Region\n \n2a - No fallow land", size=12, pad=5)
plt.ylabel("")
plt.tick_params(axis='y', which='both', labelleft=False, labelright=False,
                rotation=0, right=False, left=True)
plt.tick_params(axis='x', which='both', labelbottom=False, rotation=0)

plt.subplot(3, 3, 3)
sns.heatmap(df_super_lst[5][0].loc[::,0.1:1.0], cbar=None, cmap=cmap, center=1,
                annot=False,
            yticklabels=25,
            xticklabels=1)
plt.contour(df_super_lst[5][0].loc[::,0.1:1.0], levels=[1.0], origin = "lower",
            colors="black", linestyles="--")
plt.grid(alpha=0.33)
plt.title("Catchments\n \n3a - No fallow land", size=12, pad=5)
plt.ylabel("")
plt.tick_params(axis='y', which='both', labelleft=False, labelright=False,
                rotation=0, right=False, left=True)
plt.tick_params(axis='x', which='both', labelbottom=False, rotation=0)

plt.subplot(3, 3, 4)
sns.heatmap(df_super_lst[3][1].loc[::,0.1:1.0], cbar=None, cmap=cmap, center=1,
                annot=False,
            yticklabels=25,
            xticklabels=1)
plt.contour(df_super_lst[3][1].loc[::,0.1:1.0], levels=[1.0], origin = "lower",
            colors="black", linestyles="--")
plt.grid(alpha=0.33)
plt.title("1b - 1:2 fallow land", size=12, pad=5)
plt.ylabel("Population size", size=16, labelpad=10)
plt.yticks(size=11)
plt.tick_params(axis='x', which='both', labelbottom=False, rotation=0)

plt.subplot(3, 3, 5)
sns.heatmap(df_super_lst[4][1].loc[::,0.1:1.0], cbar=None, cmap=cmap, center=1,
                annot=False,
            yticklabels=25,
            xticklabels=1)
plt.contour(df_super_lst[4][1].loc[::,0.1:1.0], levels=[1.0], origin = "lower",
            colors="black", linestyles="--")
plt.grid(alpha=0.33)
plt.title("2b - 1:2 fallow land", size=12, pad=5)
plt.ylabel("")
plt.tick_params(axis='y', which='both', labelleft=False, labelright=False,
                rotation=0, right=False, left=True)
plt.tick_params(axis='x', which='both', labelbottom=False, rotation=0)

plt.subplot(3, 3, 6)
sns.heatmap(df_super_lst[5][1].loc[::,0.1:1.0], cbar=None, cmap=cmap, center=1,
                annot=False,
            yticklabels=25,
            xticklabels=1)
plt.contour(df_super_lst[5][1].loc[::,0.1:1.0], levels=[1.0], origin = "lower",
            colors="black", linestyles="--")
plt.grid(alpha=0.33)
plt.title("3b - 1:2 fallow land", size=12, pad=5)
plt.ylabel("")
plt.tick_params(axis='y', which='both', labelleft=False, labelright=False,
                rotation=0, right=False, left=True)
plt.tick_params(axis='x', which='both', labelbottom=False, rotation=0)

plt.subplot(3, 3, 7)
sns.heatmap(df_super_lst[3][2].loc[::,0.1:1.0], cbar=None, cmap=cmap, center=1,
                annot=False,
            yticklabels=25,
            xticklabels=1)
plt.contour(df_super_lst[3][2].loc[::,0.1:1.0], levels=[1.0], origin = "lower",
            colors="black", linestyles="--")
plt.grid(alpha=0.33)
plt.yticks(size=11)
plt.xticks(size=11)
plt.title("1c - 1:1 fallow land", size=12, pad=5)
plt.ylabel("")

plt.subplot(3, 3, 8)
sns.heatmap(df_super_lst[4][2].loc[::,0.1:1.0], cbar=None, cmap=cmap, center=1,
                annot=False,
            yticklabels=25,
            xticklabels=1)
plt.contour(df_super_lst[4][2].loc[::,0.1:1.0], levels=[1.0], origin = "lower",
            colors="black", linestyles="--")
plt.grid(alpha=0.33)
plt.xticks(size=11)
plt.title("2c - 1:1 fallow land", size=12, pad=5)
plt.ylabel("")
plt.xlabel("Suitability threshold", size=16, labelpad=10)
plt.tick_params(axis='y', which='both', labelleft=False, labelright=False,
                rotation=0, right=False, left=True)

plt.subplot(3, 3, 9)
sns.heatmap(df_super_lst[5][2].loc[::,0.1:1.0], cbar=None, cmap=cmap, center=1,
                annot=False,
            yticklabels=25,
            xticklabels=1)
plt.contour(df_super_lst[5][2].loc[::,0.1:1.0], levels=[1.0], origin = "lower",
            colors="black", linestyles="--")
plt.grid(alpha=0.33)
plt.xticks(size=11)
plt.title("3c - 1:1 fallow land", size=12, pad=5)
plt.ylabel("")
# plt.xlabel("Suitability threshold", size=16, labelpad=10)
plt.tick_params(axis='y', which='both', labelleft=False, labelright=False,
                rotation=0, right=False, left=True)

cbar_ax = fig.add_axes([.85, .1, .03, 0.8])
norm = mpl.colors.Normalize(vmin=0, vmax=2)
cb1 = mpl.colorbar.ColorbarBase(cbar_ax, cmap=cmap, norm=norm,
                                orientation='vertical')
cb1.set_label('Carrying capacity index ($CCI$)', size=16, labelpad=30, rotation=270)

# fig.suptitle('Carrying capacity estimation', size=20)
plt.savefig(workdir+"Paper/Figures/heatmap_CCI_high_labour.png",dpi=600)
#plt.show()    










